/* Copyright (c) 2016 Token Zero Ltd
 * All rights reserved.
 * 
 */

#include <stdio.h>
#include <string.h>
#include "nrf_gpio.h"
#include "nrf_drv_gpiote.h"
#include "core_service.h"
#include "ble_srv_common.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_uart.h"
#include "app_twi.h"
#include "hardware.h"
#include "lsm303.h"
#include "sht21.h"


#define DEFAULT_SERVICE_CONFIG		0x01

#define DEFAULT_TIMER_PRESCALER		255
#define BASE_TIMER_INTERVAL			0x00000200//115200
#define RAPID_TIMER_INTERVAL		APP_TIMER_TICKS(40, DEFAULT_TIMER_PRESCALER)
#define APP_TIMER_OP_QUEUE_SIZE		4


// FROM_SERVICE_TUTORIAL: Declare a service structure for our application
static ble_cs_t m_core_service;

// Declare an app_timer id variable and define our timer interval and define a timer interval
APP_TIMER_DEF(m_base_timer_id);

// Service variables
static uint32_t rx_samplerate = BASE_TIMER_INTERVAL;


uint8_t alert = 0;
uint8_t alert_cooldown = 0;


static void timer_adjust(uint32_t adj_value)
{
	uint32_t char_timer_interval = adj_value;
	printf("\n\r Timer int: %d\n\r", char_timer_interval);
	// make adjustment to timer/sample rate based on received data
	// stop timer
	app_timer_stop(m_base_timer_id);
	// restart timer with new timeout val
	app_timer_start(m_base_timer_id, char_timer_interval, NULL);

}

static void sensor_rh_characteristic_update(uint32_t * p_char_value)
{
	uint32_t err_code;
	
    // Update characteristic value
	if (m_core_service.conn_handle != BLE_CONN_HANDLE_INVALID)
	{
		uint16_t len = 1;
		
		ble_gatts_hvx_params_t hvx_params;
		memset(&hvx_params, 0, sizeof(hvx_params));

		hvx_params.handle = m_core_service.rh_handles.value_handle;
		hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
		hvx_params.offset = 0;
		hvx_params.p_len  = &len;
		hvx_params.p_data = (uint8_t*)p_char_value; //characteristic_value;  

		err_code = sd_ble_gatts_hvx(m_core_service.conn_handle, &hvx_params);
		APP_ERROR_CHECK(err_code);
	}    
	
}

static void sensor_rh_t_read_callback(ret_code_t result, void * p_data)
{
	nrf_gpio_pin_set(24);
	printf(".\n\rTWI Callback");
	
	if(result != NRF_SUCCESS)
	{
		printf("Transfer failure - twi_read_callback - Error: %d\r\n", (int)result); 
	}else{
		
		uint16_t rh_raw = (m_sht21_buffer[0]<<8) | (m_sht21_buffer[1]&0xFC) ; // 8 bit	
		uint32_t rh_conv   = -6 + ((125 * rh_raw)>>16); 	// RH= -6 + 125 * SRH/2^16 // 34us as floats, 2us as ints
		
		uint32_t temp_raw = (m_sht21_buffer[T_START_INDEX]<<8) | (m_sht21_buffer[T_START_INDEX+1]&0xFC); // 8 bit	
		float t_conv    = -46.85 + 175.72/65536 * (float)temp_raw; 	// T= -46.85 + 175.72 * ST/2^16
				
		printf("\033[1;20f RH: %d%%\n\r", rh_conv);
		printf("\033[2;20f T: %6.2fC\n\r", t_conv);
			
		sensor_rh_characteristic_update(&rh_conv);
		
	}
	nrf_gpio_pin_clear(24);
}

static void sensor_rh_t_read(void)
{
	printf("\n\rTWI Read");
	static app_twi_transfer_t transfers[] = 
	{
		SHT21_RH_READ(&m_sht21_buffer[RH_START_INDEX]),
		SHT21_T_READ(&m_sht21_buffer[T_START_INDEX])
	};
	static app_twi_transaction_t const transaction =
	{
	.callback            = sensor_rh_t_read_callback,
	.p_user_data         = NULL,
	.p_transfers         = transfers,
	.number_of_transfers = sizeof(transfers) / sizeof(transfers[0])
	};
	
	transact_twi(&transaction);
}


// This is a timer event handler
static void base_timer_timeout_handler(void * p_context)
{
	sensor_rh_t_read();
	
	// Sample accelerometer
	// Sample RH and temp
	// wait for results and Update characteristics

}


/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module. This creates and starts application timers.
 */
static void timer_init(void)
{
    // Initialize timer module.
    APP_TIMER_INIT(DEFAULT_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
    // Initiate our timer
    app_timer_create(&m_base_timer_id, APP_TIMER_MODE_REPEATED, base_timer_timeout_handler);
	app_timer_start(m_base_timer_id, BASE_TIMER_INTERVAL, NULL);
	
}



/**@brief Function for handling BLE Stack events related to our service and characteristic.
 *
 * @details Handles all events from the BLE stack of interest to 'Our Service'.
 *
 * @param[in]   p_core_service       Our Service structure.
 * @param[in]   p_ble_evt        	Event received from the BLE stack.
 *
 */
void ble_core_service_on_ble_evt(ble_evt_t * p_ble_evt)
{
    // Implement switch case handling BLE events related to our service. 
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            m_core_service.conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            break;
        case BLE_GAP_EVT_DISCONNECTED:
            m_core_service.conn_handle = BLE_CONN_HANDLE_INVALID;
        case BLE_GATTS_EVT_WRITE:
            break;
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for initiating our new service.
 *
 * @param[in]   p_core_service        Our Service structure.
 *
 */
void core_service_init(void)
{
   

    uint32_t            err_code;
	
    ble_uuid_t          service_uuid;
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr;
    ble_gatts_attr_md_t attr_md;
    ble_gatts_attr_md_t cccd_md;
    ble_gatts_attr_md_t char_ud_md;

    static uint8_t peripheral_data;
    static uint8_t peripheral_ud[] = "Modifiable multilink_peripheral Data";
	
	m_core_service.conn_handle = BLE_CONN_HANDLE_INVALID;

    ble_uuid128_t base_uuid = BLE_UUID_CORE_BASE_UUID;

    err_code = sd_ble_uuid_vs_add(&base_uuid, &service_uuid.type);
    APP_ERROR_CHECK(err_code);

    service_uuid.uuid = BLE_UUID_CORE_SERVICE;

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, 
										&service_uuid, 
										&m_core_service.service_handle);
    APP_ERROR_CHECK(err_code);
	
	// characteristic setup //

    service_uuid.uuid = BLE_UUID_RH_CHAR_UUID;

    memset(&attr, 0, sizeof(ble_gatts_attr_t));
    attr.p_uuid    = &service_uuid;
    attr.p_attr_md = &attr_md;
    attr.max_len   = 1;
    attr.p_value   = &peripheral_data;
    attr.init_len  = sizeof(peripheral_data);

    memset(&attr_md, 0, sizeof(ble_gatts_attr_md_t));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&attr_md.write_perm);
    attr_md.vloc = BLE_GATTS_VLOC_STACK;
    attr_md.vlen = 0;

    memset(&cccd_md, 0, sizeof(ble_gatts_attr_md_t));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_ENC_NO_MITM(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(ble_gatts_char_md_t));
    char_md.p_cccd_md               = &cccd_md;
    char_md.char_props.notify       = 1;
    char_md.char_props.indicate     = 1;
    char_md.char_props.read         = 1;
    char_md.char_props.write        = 1;
    char_md.char_ext_props.wr_aux   = 1;
    char_md.p_user_desc_md          = &char_ud_md;
    char_md.p_char_user_desc        = peripheral_ud;
    char_md.char_user_desc_size     = (uint8_t)strlen((char *)peripheral_ud);
    char_md.char_user_desc_max_size = (uint8_t)strlen((char *)peripheral_ud);

    memset(&char_ud_md, 0, sizeof(ble_gatts_attr_md_t));
    char_ud_md.vloc = BLE_GATTS_VLOC_STACK;
    char_ud_md.vlen = 1;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&char_ud_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&char_ud_md.write_perm);

    err_code = sd_ble_gatts_characteristic_add(m_core_service.service_handle,
                                               &char_md,
                                               &attr,
                                               &m_core_service.rh_handles);
	APP_ERROR_CHECK(err_code);										   
											
	// initialise timer
	timer_init();
}


