/* Copyright (c) 2015 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 * @defgroup tw_sensor_example main.c
 * @{
 * @ingroup nrf_twi_example
 * @brief TWI Sensor Example main file.
 *
 * This file contains the source code for a sample application using TWI.
 *
 */

#include <stdio.h>
#include "hardware.h"
#include "SHT21.h"
#include "nrf_gpio.h"
#include "app_util_platform.h"
#include "app_error.h"
#include "app_uart.h"
#include "nrf_adc.h"
#include "nrf_delay.h"



#define MAX_PENDING_TRANSACTIONS		100
#define UART_TX_BUF_SIZE 				256
#define UART_RX_BUF_SIZE 				1

static app_twi_t m_app_twi = APP_TWI_INSTANCE(0); //bak1

void transact_twi(app_twi_transaction_t const * p_transaction)
{
	uint32_t err_code;
	
	err_code = app_twi_schedule(&m_app_twi, p_transaction);
	APP_ERROR_CHECK(err_code);
}

/**
 * @brief Union to keep raw and converted data from accelerometer samples at one memory space.
 */
typedef union{
    uint8_t raw;
    int8_t  conv;
}elem_t;

/**
 * @brief Structure for holding samples from accelerometer.
 */
typedef struct
{
	uint8_t  x_l;
	uint8_t  x_h;
	uint8_t  y_l;
	uint8_t  y_h;
	uint8_t  z_l;
	uint8_t  z_h;
} acc_sample_t;

//temp count register
int count = 0;
int stamp = 0;

/** other functions, might be moved to other 
files to complement header files(LSM303.c / SHT21.c) **/

/* LSM303 function prototypes */
void LSM303_read(void);
//void LSM303_read_acc(acc_sample_t *p_new_sample);

/* TWI function prototypes */
void twi_read_callback(ret_code_t result, void * p_data);

/**
 * @brief UART events handler.
 */

static void uart_events_handler(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        case APP_UART_COMMUNICATION_ERROR:
            //APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        default:
            break;
    }
}

/**
 * @brief UART initialization.
 */
static void uart_config(void)
{
    uint32_t err_code;
	
    const app_uart_comm_params_t comm_params =
    {
        UART_RX_PIN,
        UART_TX_PIN,
        19,
        20,
        APP_UART_FLOW_CONTROL_DISABLED,
        false,
        UART_BAUDRATE_BAUDRATE_Baud115200
    };

    APP_UART_FIFO_INIT(&comm_params,
                       UART_RX_BUF_SIZE,
                       UART_TX_BUF_SIZE,
                       uart_events_handler,
                       APP_IRQ_PRIORITY_LOW,
                       err_code);

    APP_ERROR_CHECK(err_code);
}

	
/**
 * @brief TWI initialisation function
 */
static void twi_init(void)
{
	ret_code_t err_code;
	
	nrf_drv_twi_config_t const config = {
		.scl						= I2C_SCL,
		.sda						= I2C_SDA,
		.frequency					= NRF_TWI_FREQ_100K,
		.interrupt_priority			= APP_IRQ_PRIORITY_LOW
	};
	
	APP_TWI_INIT(&m_app_twi, &config, MAX_PENDING_TRANSACTIONS, err_code);
	APP_ERROR_CHECK(err_code);
}


/**
 * @brief ADC initialization.
 */
static void adc_init(void)
{

	nrf_adc_config_t adc_config = NRF_ADC_CONFIG_DEFAULT;
    adc_config.scaling = ADC_CONFIG_INPSEL_AnalogInputOneThirdPrescaling;
    nrf_adc_configure(&adc_config); 
    nrf_adc_int_enable(ADC_INTENSET_END_Msk);
    NVIC_EnableIRQ(ADC_IRQn);
    NVIC_SetPriority(ADC_IRQn, 3);
    nrf_adc_input_select(NRF_ADC_CONFIG_INPUT_3);
	
	
}

/* Interrupt handler for ADC data ready event */
void ADC_IRQHandler(void)
{
    uint16_t adc_value;
    nrf_adc_conversion_event_clean();
    nrf_adc_stop();
    
    adc_value = nrf_adc_result_get();
	
	printf("\033[1;40f L: %d  ", adc_value);
    
}

void adc_light_read(void){
	
	nrf_adc_start();
	
}


/**
 * @brief Function for main hardware configuration and initialisation.
 */
void hardware_init(void)
{
	
	nrf_gpio_cfg_output(LED);
	nrf_gpio_pin_clear(LED);

	uart_config();	// configure UART
	
	// Unnecessary connection text //
	//printf("\033[2J\033[1;1H");
	printf("\n\r -- Sensor connection established --\n\r");

	
	twi_init();		// I2C initialise

	
	// Configure SHT21 (default)
	printf("\n\r Configuring SHT21...");
	SHT21_get_register(&m_app_twi, &m_sht21_buffer[4]);
	//printf("\n\r 0x%x -> ", m_sht21_buffer[4]);
	SHT21_default_config(&m_app_twi, &m_sht21_buffer[4]);
	SHT21_get_register(&m_app_twi, &m_sht21_buffer[4]);
	//printf("0x%x", m_sht21_buffer[4]);

	/*
	printf("\n\r Configuring ADC...");
	adc_init();		// Initialise the ADC for light level reading (and later battery?)
	*/

	printf("\n\r -- Sensor Config complete -- \n\r");

}

/** @} */
